/*
 * Copyright (c) 2007, Novell Inc.
 *
 * This program is licensed under the BSD license, read LICENSE.BSD
 * for further information
 */

/*
 * poolarch.c
 *
 * create architecture policies
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "pool.h"
#include "poolid.h"
#include "poolarch.h"
#include "util.h"

/* Provide the const char *archpolicies structure */
#include "core-arch.h"

void
pool_setarch(Pool *pool, const char *arch)
{
  const char *a;
  unsigned int score = 0x10001;
  size_t l;
  char d;
  int i;
  Id *id2arch;
  Id id, lastarch;

  pool->id2arch = sat_free(pool->id2arch);
  pool->id2color = sat_free(pool->id2color);
  if (!arch)
    {
      pool->lastarch = 0;
      return;
    }
#if defined(DEBIAN_SEMANTICS) || defined(OE_CORE_SEMANTICS)
  id = ARCH_ALL;
#else
  id = ARCH_NOARCH;
#endif
  lastarch = id + 255;
  id2arch = sat_calloc(lastarch + 1, sizeof(Id));
  id2arch[id] = 1;

  a = "";
  for (i = 0; archpolicies[i]; i += 2)
    if (!strcmp(archpolicies[i], arch))
      break;
  if (archpolicies[i])
    a = archpolicies[i + 1];
  d = 0;
  while (*a)
    {
      l = strcspn(a, ":=>");
      if (l)
	{
	  id = strn2id(pool, a, l, 1);
	  if (id > lastarch)
	    {
	      id2arch = sat_realloc(id2arch, (id + 255 + 1) * sizeof(Id));
	      memset(id2arch + lastarch + 1, 0, (id + 255 - lastarch) * sizeof(Id));
	      lastarch = id + 255;
	    }
	  if (id2arch[id] == 0)
	    {
	      if (d == ':')
		score += 0x10000;
	      else if (d == '>')
		score += 0x00001;
	      id2arch[id] = score;
	    }
	}
      a += l;
      if ((d = *a++) == 0)
	break;
    }
  pool->id2arch = id2arch;
  pool->lastarch = lastarch;
}

unsigned char
pool_arch2color_slow(Pool *pool, Id arch)
{
  const char *s;
  unsigned char color;

  if (arch > pool->lastarch)
    return ARCHCOLOR_ALL;
  if (!pool->id2color)
    pool->id2color = sat_calloc(pool->lastarch + 1, 1);
  s = id2str(pool, arch);
  if (arch == ARCH_NOARCH || arch == ARCH_ALL)
    color = ARCHCOLOR_ALL;
  else if (!strcmp(s, "s390x") || strstr(s, "64"))
    color = ARCHCOLOR_64;
  else
    color = ARCHCOLOR_32;
  return color;
}

